"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const https = __importStar(require("node:https"));
const node_path_1 = require("node:path");
class PluginTelegramWebhook {
    constructor(sls, _, ctx) {
        this.sls = sls;
        this.ctx = ctx;
        this.hooks = {
            "after:deploy:deploy": this.setWebhook.bind(this),
            "after:remove:remove": this.removeWebhook.bind(this),
        };
    }
    async setWebhook() {
        this.validateWebhookConfig();
        const { webhook } = this.resolveConfig();
        switch (webhook.type) {
            case "function":
                await this.setWebhookFunction(webhook.value);
                break;
            case "path":
                await this.setWebhookPath(webhook.value);
                break;
            default:
                throw new Error("[serverless-plugin-telegram-webhook] Invalid webhook type");
        }
    }
    async removeWebhook() {
        this.ctx.log.notice("[serverless-plugin-telegram-webhook] Removing webhook url from https://telegram.org");
        const res = await this.apiCall("setWebhook", { url: "" });
        if ((res === null || res === void 0 ? void 0 : res.ok) === false) {
            this.ctx.log.error("[serverless-plugin-telegram-webhook] https://api.telegram.org Remove webhook url failed with response: ");
            this.ctx.log.error(res);
            return;
        }
        this.ctx.log.success(`[serverless-plugin-telegram-webhook] Successfully removed webhook url from https://api.telegram.org`);
        this.ctx.log.info("Response from https://api.telegram.org: ");
        this.ctx.log.info(res);
    }
    async setWebhookFunction(lambda) {
        var _a;
        const func = this.sls.service.getFunction(lambda);
        if (!func)
            throw new Error(`[serverless-plugin-telegram-webhook] self:functions.${lambda} not found`);
        const event = (func.events || []).find((event) => event.http || event.httpApi);
        if (!event)
            throw new Error(`[serverless-plugin-telegram-webhook] self:functions.${lambda} has no http or httpApi event attached to it`);
        const path = (_a = (event.http || event.httpApi)) === null || _a === void 0 ? void 0 : _a.path;
        if (!path)
            throw new Error(`[serverless-plugin-telegram-webhook] self:functions.${lambda} http/httpApi event is missing the "path" property`);
        return await this.setWebhookPath(path);
    }
    async setWebhookPath(path) {
        const endpoint = await this.resolveServiceEndpoint();
        if (!endpoint)
            return;
        const url = new URL((0, node_path_1.join)(this.sls.service.provider.stage, path), endpoint).href;
        const res = await this.apiCall("setWebhook", { url });
        if ((res === null || res === void 0 ? void 0 : res.ok) === false) {
            this.ctx.log.error("[serverless-plugin-telegram-webhook] https://api.telegram.org setWebhook API call failed with response: ");
            this.ctx.log.error(res);
            return;
        }
        this.ctx.log.success(`[serverless-plugin-telegram-webhook] Successfully set https://api.telegram.org bot webhook to ${url}`);
        this.ctx.log.info("Response from https://api.telegram.org: ");
        this.ctx.log.info(res);
    }
    async resolveServiceEndpoint() {
        var _a;
        const aws = this.sls.getProvider("aws");
        const res = await aws.request("CloudFormation", "describeStacks", {
            StackName: aws.naming.getStackName(),
        });
        if (!res || !((_a = res.Stacks) === null || _a === void 0 ? void 0 : _a.length)) {
            throw new Error(`[serverless-plugin-telegram-webhook] Stack: ${aws.naming.getStackName()} not found`);
        }
        const outputs = res.Stacks[0].Outputs;
        let serviceEndpointOutput = outputs.find(({ OutputKey }) => OutputKey === "ServiceEndpoint");
        if (!serviceEndpointOutput) {
            throw new Error(`[serverless-plugin-telegram-webhook] Stack: ${aws.naming.getStackName()} does not have a service endpoint attached (i.e no lambda is triggered by http/httpApi)`);
        }
        return serviceEndpointOutput.OutputValue;
    }
    async apiCall(method, body) {
        return new Promise((resolve, reject) => {
            let chunks = [];
            const opts = {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                },
            };
            const req = https.request(this.resolveMethodUrl(method), opts, (res) => {
                res.on("error", (error) => reject(error));
                res.on("data", (chunk) => {
                    chunks.push(chunk);
                });
                res.on("end", () => {
                    const buffer = Buffer.concat(chunks);
                    try {
                        resolve(JSON.parse(buffer.toString()));
                    }
                    catch (_a) {
                        resolve(buffer.toString());
                    }
                });
            });
            if (body) {
                req.write(JSON.stringify(body));
            }
            req.end();
        });
    }
    resolveMethodUrl(method) {
        const token = this.resolveToken();
        return `https://api.telegram.org/bot${token}/${method}`;
    }
    resolveConfig() {
        const config = this.sls.service.custom;
        if (config) {
            if (config["telegramWebhook"])
                return config["telegramWebhook"];
            if (config["telegram-webhook"])
                return config["telegram-webhook"];
            if (config["telegram_webhook"])
                return config["telegram_webhook"];
            if (config["telegramwebhook"])
                return config["telegramwebhook"];
        }
        throw new Error("Unable to resolve serverless-plugin-telegram-webhook config in self:custom");
    }
    resolveToken() {
        const conf = this.resolveConfig();
        if (conf["token"])
            return conf["token"];
        throw new Error("Unable to resolve serverless-plugin-telegram-webhook token (telegram bot token)");
    }
    validateWebhookConfig() {
        const conf = this.resolveConfig();
        if (!conf["webhook"] || !conf["webhook"].type || !conf["webhook"].value)
            throw new Error("Unable to resolve serverless-plugin-telegram-webhook webhook config");
    }
}
module.exports = PluginTelegramWebhook;
//# sourceMappingURL=plugin.js.map